package com.danmassey.rugbe;

import java.util.LinkedHashMap;

import com.danmassey.rugbe.board.Board;
import com.danmassey.rugbe.board.Pkg;
import com.danmassey.rugbe.brain.BrainFactory;
import com.danmassey.rugbe.brain.IBrain;
import com.danmassey.rugbe.interpreter.ResponseInterpreter;
import com.danmassey.rugbe.net.ComChannel;
import com.danmassey.rugbe.parsers.ProtocolParser;
import com.danmassey.rugbe.robot.Robot;
import com.danmassey.rugbe.ui.DebugUI;

/**
 * Main game class
 * 
 * @author dan
 * Created on Aug 30, 2002
 **/
public class Rugbe {
	private ComChannel com;	
	private DebugUI ui;
	private ProtocolParser parser;
	private boolean showUI;
	
	/**
	 * Command line args:<br/>
	 * host port [brain_config_file] [-ui] <br/>
	 **/
	public static void main(String[] args) {
		Rugbe rugbe = new Rugbe(args[0], args[1]);
		
		// Handles host config
		String brainConfig;
		if (args.length > 2) {
			brainConfig = args[2];
		} else {
			brainConfig = null;
		}
		
		// Handles ui
		if (args.length > 2) {
			if("-ui".equals(args[3])) {
				rugbe.showUI = true;
			} else {
				rugbe.showUI = false;
			}
		}
		
		rugbe.play(brainConfig);
	}
	
	public Rugbe(String host, String port) {
		this.com = new ComChannel(host, port);	
		this.parser = new ProtocolParser();
	}

	/**
	 * Main play loop. Initializes the world model. Then starts the brain
	 * thinking.
	 **/
	public void play(String brainConfig) {
		this.com.init();
		Board board = parser.parseBoard(com);
		System.out.println(board.toString());
		Robot robot = parser.parseRobot(com);
		System.out.println(robot.toString());
		LinkedHashMap nbots = new LinkedHashMap(32, 0.5f);
		ResponseInterpreter interpreter = new ResponseInterpreter();
		interpreter.run(com.readLine(), robot, nbots, new Pkg[] {null});
		if (showUI) {
			initUI(robot, board, nbots);
			this.ui.drawWorld();
		}
		IBrain brain = BrainFactory.makeBrain(brainConfig, robot, board, nbots, com, ui);
		
		boolean alive = true;
		while (alive) {
			Pkg[] packages = parser.parsePackages(com);
			if (showUI) {
				this.ui.displayPackages(packages);
			}
			brain.process(packages);
			alive = interpreter.run(com.readLine(), robot, nbots, packages);
			if (showUI) {
				this.ui.drawWorld();
			}
		}
		System.out.println("DEAD");
		System.exit(0);
	}
	
	/**
	 * Method initUI.
	 */
	private void initUI(Robot robot, Board board, LinkedHashMap nbots) {
		this.ui = new DebugUI(robot, board, nbots);
		this.ui.show();
	}
}
