package com.danmassey.rugbe.brain.path;

import java.util.LinkedHashMap;
import java.util.TreeMap;

import com.danmassey.rugbe.board.*;
import com.danmassey.rugbe.board.Board;

/**
 * @author dmassey
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class AStar {
	private Board board;
	
	public AStar(Board board) {
		this.board = board;
		// Consider calculating a terrain weighting factor to add to the heuristic
	}
	
	public Node findPath(int startX, int startY, int goalX, int goalY) {
		TreeMap open = new TreeMap();
		TreeMap closed = new TreeMap();
		Node goal = new Node(goalX, goalY);
		Node start = new Node(startX, startY);	
		
		if (start.sameNode(goal)) {
			return goal;
		}
		
		start.setG(0);
		start.setH(start.distance(goal));
		start.setF(start.getG() + start.getH());
		start.setParent(null);
		open.put(start, start);
		
		while (!open.isEmpty()){
			Node n = (Node) open.firstKey();
			//System.out.println(n.toString());
			open.remove(n);
			Node[] nextNodes = possibleSuccessors(n);
			for (int i = 0; i < nextNodes.length; i++) {
				if (nextNodes[i].sameNode(goal)) {
					nextNodes[i].setParent(n);
					return constructPath(nextNodes[i]);
				}
				int d = 1;
				if (n.getTerrain() == Board.BUFFER_1) {
					d = 5;
				} else if (n.getTerrain() == Board.BUFFER_2) {
					d = 3;
				}
				int newg = n.getG() + d;
				Node old = null;
				if (open.containsValue(nextNodes[i]) || closed.containsValue(nextNodes[i])) {
					old = nextNodes[i].getTemp();
					nextNodes[i].clearTemp();
					if (old.getG() <= newg) {
							continue;
					} else {
						open.remove(old);
						closed.remove(old);
					}
				}
				nextNodes[i].setParent(n);
				nextNodes[i].setG(newg);
				nextNodes[i].setH(nextNodes[i].distance(goal));
				nextNodes[i].setF(nextNodes[i].getG() + nextNodes[i].getH());
				open.put(nextNodes[i], nextNodes[i]);
			}			
			closed.put(n, n);
		}
				
		return null;		
	}

	/**
	 * Method possibleSuccessors.
	 * @param n
	 * @return Node[]
	 */
	private Node[] possibleSuccessors(Node n) {
		char[] terrain = this.board.getSurroundingTiles(n.getX(), n.getY());
		int good = 0;
		for (int i = 0; i < 4; i++) {
			if ((terrain[i] == Board.PLAIN) || (terrain[i] == Board.HOME)) {
				good++;
			}
		}
		Node[] nodes = new Node[good];
		int j = 0;
		for (int i = 0; i < 4; i++) {
			if ((terrain[i] == Board.PLAIN) || (terrain[i] == Board.HOME) 
					|| (terrain[i] == Board.BUFFER_1) || (terrain[i] == Board.BUFFER_2)) {
				switch (i) {
					case 0:	// North
						nodes[j++] = new Node(n.getX(), n.getY() + 1, terrain[i]);
						break;
						
					case 1:	// East
						nodes[j++] = new Node(n.getX() + 1, n.getY(), terrain[i]);
						break;
						
					case 2:	// South
						nodes[j++] = new Node(n.getX(), n.getY() - 1, terrain[i]);
						break;
						
					case 3:	// West
						nodes[j++] = new Node(n.getX() - 1, n.getY(), terrain[i]);
						break;
				}
			} 
		}
		return nodes;
	}


	/**
	 * Method constructPath.
	 * @return Node[]
	 */
	private Node constructPath(Node end) {
		Node n = end;
		Node p = n.getParent();
		while (p != null) {
			p.setChild(n);
			n = p;
			p = n.getParent();
		}
		//printPath(n);
		return n.getChild();
	}
	
	private void printPath(Node n) {
		Node path = n;
		while (path != null) {
			System.out.println("Path > " + path.toString());
			path = path.getChild();
		}
	}
}
