package com.danmassey.rugbe.interpreter;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.StringTokenizer;

import com.danmassey.rugbe.board.Pkg;
import com.danmassey.rugbe.robot.NBot;
import com.danmassey.rugbe.robot.Robot;

/**
 * 
 * 
 * @author dan
 * Created on Aug 30, 2002
 **/
public class ResponseInterpreter {
	private int turnCounter = 0;
		
	/**
	 * Method run.
	 * @param string
	 * @param nbots
	 */
	public boolean run(String response, Robot robot, LinkedHashMap nbots, Pkg[] packages) {
		NBot currentBot = null;
		System.out.println(turnCounter++ + ") " + response);
				
		StringTokenizer tokens = new StringTokenizer(response);
		String token;
		
		// Primary interpreter loop -- performs no error detection/correction
		while (tokens.hasMoreTokens()) {
			
			token = tokens.nextToken();
			
			switch (token.charAt(0)) {
				case 'N':	// North
					currentBot.moveNorth();
					break;
					
				case 'S':	// South
					currentBot.moveSouth();
					break;
					
				case 'E':	// East
					currentBot.moveEast();
					break;
					
				case 'W':	// West
					currentBot.moveWest();
					break;
					
				case 'P':	// Pickup
					int pickPkgId = Integer.parseInt(tokens.nextToken());
					currentBot.pickUp();
					if (currentBot.isMe()) {
						robot.pickUp(pickPkgId, packages);
					}
					break;
					
				case 'D':	// Drop
					int dropPkgId = Integer.parseInt(tokens.nextToken());
					currentBot.drop();
					if (currentBot.isMe()) {
						robot.drop(dropPkgId);
					}
					break;
					
				case 'X':	// Specific point
					currentBot.setX(Integer.parseInt(tokens.nextToken()));
					tokens.nextToken();	// Throws away Y
					currentBot.setY(Integer.parseInt(tokens.nextToken()));					
					break;
					
				case '#':	// No update, does nothing
					currentBot = loadNBot(token, robot, nbots);
					break;
					
				case 'R':	// Check for death
					// "Robot XX died."
					return false;
					
				case '*':	// *** Reason: Game over, etc.
					return false;
					
				default:	// Should never happen
					System.out.println("Unrecognized token: " + token);
					// do nothing -- better to drift than to bottom out
			}
			
			// set the updated flag
			currentBot.setUpdated(true);
		}			
		return cleanup(nbots);
	}

	/**
	 * Loads the NBot from the collection. Creates a new NBot if one with the
	 * given id does not already exist. Assigns correct NBot to local Robot.
	 **/
	private NBot loadNBot(String idString, Robot robot, LinkedHashMap nbots) {
		NBot currentBot = (NBot) nbots.get(idString);
		if (currentBot == null) {
			int id = Integer.parseInt(idString.substring(1));
			boolean isMe = false;
			if (robot.getId() == id) {
				isMe = true;
			}
			currentBot = new NBot(id, 0, 0, isMe);
			nbots.put(idString, currentBot);
			if (isMe) {
				robot.setNbot(currentBot);
			}
		}
		return currentBot;
	}
	
	/**
	 * Method cleanup.
	 * @param nbots
	 * @return boolean
	 */
	private boolean cleanup(LinkedHashMap nbots) {
		boolean alive = true;
		Iterator iter = nbots.values().iterator();
		while (iter.hasNext()) {
			NBot nbot = (NBot) iter.next();
			if (nbot.isUpdated()) {
				nbot.setUpdated(false);
			} else {
				iter.remove();
				System.out.println("Bot: " + nbot.getId() + " is dead.");
				if (nbot.isMe()) {
					alive = false;
				}
			}
		}
			
		return alive;
	}
}
