package com.danmassey.rugbe.net;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.Socket;
import java.net.UnknownHostException;

/**
 * Provides network communication services.
 * 
 * @author dan
 * Created on Aug 30, 2002
 **/
public class ComChannel {
	private Socket sock;
	private BufferedReader reader;
	private BufferedWriter writer;
	public static final String NORTH = "N";
	public static final String SOUTH = "S";
	public static final String EAST = "E";
	public static final String WEST = "W";
	
	private static final String INIT_STRING = "Player";
	
	/**
	 * Prepares the socket communications for the client.
	 **/
	public ComChannel(String host, String port) {
		try {
			this.sock = new Socket(host, new Integer(port).intValue());
			this.reader = new BufferedReader(
					new InputStreamReader(this.sock.getInputStream()));
			this.writer = new BufferedWriter(
					new OutputStreamWriter(this.sock.getOutputStream()));
		} catch (UnknownHostException e) {
			System.out.println("Unknown host: " + host + "  port: " + port);
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("IOException");
			e.printStackTrace();
		}
	}
		

	/**
	 * Sends initial client string to server.
	 **/
	public void init() {
		try {
			this.writer.write(INIT_STRING + "\n");
			this.writer.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @return a single line from the incoming ascii text stream.
	 **/
	public String readLine() {
		try {
			String incoming = this.reader.readLine();
			while (incoming == null) {
				incoming = this.reader.readLine();
			} 
			return incoming;
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(42);
		}
		return null;
	}
	
	public void move(String direction, int bid) {
		String command = bid + " Move " + direction;
		send(command);
	}
	
	public void pick(int[] packageIds, int bid) {
		StringBuffer buffy = new StringBuffer();
		buffy.append(bid);
		buffy.append(" Pick");
		for (int i = 0; i < packageIds.length; i++) {
			buffy.append(" ");
			buffy.append(packageIds[i]);
		}
		send(buffy.toString());
	}
	
	public void drop(int[] packageIds, int bid) {
		StringBuffer buffy = new StringBuffer();
		buffy.append(bid);
		buffy.append(" Drop");
		for (int i = 0; i < packageIds.length; i++) {
			buffy.append(" ");
			buffy.append(packageIds[i]);
		}
		send(buffy.toString());
	}

	/**
	 * Method send.
	 * @param command
	 */
	private void send(String command) {
		System.out.println(command);
		try {
			this.writer.write(command + "\n");
			this.writer.flush();
		} catch (IOException e) {
			e.printStackTrace();
		}	
	}

}
