package com.danmassey.rugbe.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Iterator;
import java.util.LinkedHashMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import com.danmassey.rugbe.board.Board;
import com.danmassey.rugbe.board.Node;
import com.danmassey.rugbe.board.Pkg;
import com.danmassey.rugbe.robot.NBot;
import com.danmassey.rugbe.robot.Robot;

/**
 * 
 * 
 * @author dan
 * Created on Aug 31, 2002
 **/
public class DebugUI {
	private JFrame frame;
	private JLabel robotIdLabel;
	private JLabel robotCapacityLabel;
	private JLabel robotMoneyLabel;
	private JTextArea robotPackagesText;
	private JTextArea newPackagesText;
	private JPanel packagePanel;
	private JPanel worldMapPane;
	private Robot robot;
	private Board board;
	private LinkedHashMap nbots;
	private Node path = null;
	
	
	public DebugUI(Robot robot, Board board, LinkedHashMap nbots) {
		this.frame = new JFrame("Rugbe Package Delivery Bot");
		this.robot = robot;
		this.board = board;
		this.nbots = nbots;
		
		// Robot stats and packages
		JPanel robotPane = new JPanel();
		robotPane.setLayout(new BoxLayout(robotPane, BoxLayout.Y_AXIS));
		robotIdLabel = new JLabel();
		robotIdLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		robotCapacityLabel = new JLabel();
		robotCapacityLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		robotMoneyLabel = new JLabel();
		robotMoneyLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		robotPackagesText = new JTextArea();
		robotPackagesText.setAlignmentX(Component.LEFT_ALIGNMENT);
		robotPane.add(robotIdLabel);
		robotPane.add(Box.createRigidArea(new Dimension(0, 5)));
		robotPane.add(robotCapacityLabel);
		robotPane.add(Box.createRigidArea(new Dimension(0, 5)));
		robotPane.add(robotMoneyLabel);
				
		// Primary display pane and new packages		
		GridBagLayout gblayout = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		JPanel mainPane = new JPanel();
		mainPane.setLayout(gblayout);
		worldMapPane = new MapPanel(this);	
		JScrollPane mapScrollPane = new JScrollPane(worldMapPane);	
		c.insets = new Insets(0, 0, 5, 5);
		c.weightx = .8;
		c.weighty = .8;
		c.gridx = 0;
		c.gridy = 0;
		c.gridheight = 6;
		c.gridwidth = 6;
		gblayout.addLayoutComponent(mapScrollPane, c);
		mainPane.add(mapScrollPane);
		newPackagesText = new JTextArea();
		JScrollPane packageScrollPane = new JScrollPane(newPackagesText);
		JScrollPane robotPackageScrollPane = new JScrollPane(robotPackagesText);
		packagePanel = new JPanel(new GridLayout());
		packagePanel.add(packageScrollPane);
		packagePanel.add(robotPackageScrollPane);
		c.insets = new Insets(0, 0, 0, 0);
		c.weightx = .2;
		c.weighty = .2;
		c.gridx = 0;
		c.gridy = 7;
		c.gridheight = 1;
		c.gridwidth = 8;
		gblayout.addLayoutComponent(packagePanel, c);
		mainPane.add(packagePanel);
		c.insets = new Insets(0, 0, 5, 0);
		c.weightx = .2;
		c.weighty = .2;
		c.gridx = 7;
		c.gridy = 0;
		c.gridheight = 6;
		c.gridwidth = 1;
		gblayout.addLayoutComponent(robotPane, c);
		mainPane.add(robotPane);
		
		
		this.frame.getContentPane().add(mainPane);
		this.frame.setSize(1280, 1024);
		this.frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				System.exit(0);
			}
		});
		
	}
	
	/**
	 * Method show.
	 */
	public void show() {
		this.frame.setVisible(true);
	}

	/**
	 * Method showPackages.
	 * @param packages
	 */
	public void displayPackages(Pkg[] packages) {
		if (packages.length > 0) {
			StringBuffer buffy = new StringBuffer();
			for (int i = 0; i < packages.length; i++) {
				buffy.append(packages[i].toString());
				buffy.append("\n");
			}
			newPackagesText.setText(buffy.toString());
		}
	}

	/**
	 * Method drawWorld.
	 * @param robot
	 * @param board
	 * @param nbots
	 */
	public void drawWorld() {
		drawRobotPane();
		drawWorldMap();
	}
	
	private void drawRobotPane() {
		robotIdLabel.setText(robot.getId() + "");
		robotCapacityLabel.setText(robot.getCapacity() + "");
		robotMoneyLabel.setText(robot.getMoney() + "");
		Pkg pkg = robot.getPackages();
		StringBuffer buffy = new StringBuffer();
		while (pkg != null) {
			buffy.append(pkg.toString());
			buffy.append("\n");
			pkg = pkg.getNextPkg();
		}
		robotPackagesText.setText(buffy.toString());
	}
	
	private void drawWorldMap() {
		worldMapPane.setPreferredSize(new Dimension(board.getwidth() * 16 + 6,
				board.getHeight() * 16 + 6));
		Graphics g = worldMapPane.getGraphics();
		drawWorldMapGraphics(g);
	}
		
	private void drawWorldMapGraphics(Graphics g) {
		int width = board.getwidth();
		int height = board.getHeight();
		char[] grid = board.getGrid();
		int length = grid.length;
		
		
		// Draws base map
		g.setColor(Color.BLACK);
		Font monoFont = new Font("Monospaced", Font.PLAIN, 16);
		g.setFont(monoFont);
		g.clearRect(0, 0, worldMapPane.getWidth(), worldMapPane.getHeight());
		int baseY = worldMapPane.getHeight() - 3;
		int baseX = 3;		
		FontMetrics monoFontMetrics = g.getFontMetrics();
		int charWidth = monoFontMetrics.charWidth('c');
		for (int i = 0; i < height; i++) {
			g.drawChars(grid, (i * width), width, baseX, baseY - (i * 16));
		}
		
		// Draws path plot
		Node step = path;
		g.setColor(Color.YELLOW);
		char[] pt = {'*'};
		while (step != null) {
			int xCoord = step.getX();
			int yCoord = step.getY();
			int x = baseX + ((xCoord - 1) * charWidth);
			int y = baseY - ((yCoord - 1) * 16);
			g.drawChars(pt, 0, 1, x, y);
			step = step.getChild();				
		}
		
		// Draws robots
		Iterator iter = nbots.values().iterator();
		char[] botIcons = {'\u263B', '\u263A'};
		g.setColor(Color.RED);
		while(iter.hasNext()) {
			NBot nbot = (NBot) iter.next();
			int xCoord = nbot.getX();
			int yCoord = nbot.getY();
			int x = baseX + ((xCoord - 1) * charWidth);
			int y = baseY - ((yCoord - 1) * 16);
			if (nbot.isMe()) {
				g.setColor(Color.BLUE);
				g.drawChars(botIcons, 1, 1, x, y);
				g.setColor(Color.RED);
			} else {
				g.drawChars(botIcons, 0, 1, x, y);
			}			
		}				
	}
	/**
	 * Method drawWorldGraphics.
	 * @param g
	 */
	public void drawWorldGraphics(Graphics g) {
		drawWorldMapGraphics(g);
	}

	/**
	 * Method plotPath.
	 * @param path
	 */
	public void plotPath(Node path) {
		this.path = path;
	}

}
